@@ -64,11 +64,7 @@ stripped_char (gunichar ch)
 {
 	gunichar retval = 0;
 	GUnicodeType utype;
-#if GLIB_CHECK_VERSION (2,29,90)
 	gunichar decomp[4];
-#else
-	gunichar *decomp;
-#endif
 	gsize dlen;
 
 	utype = g_unichar_type (ch);
@@ -78,11 +74,7 @@ stripped_char (gunichar ch)
 	case G_UNICODE_FORMAT:
 	case G_UNICODE_UNASSIGNED:
 	case G_UNICODE_NON_SPACING_MARK:
-#if GLIB_CHECK_VERSION (2,29,90)
 	case G_UNICODE_SPACING_MARK:
-#else
-	case G_UNICODE_COMBINING_MARK:
-#endif
 	case G_UNICODE_ENCLOSING_MARK:
 		/* Ignore those */
 		break;
@@ -112,18 +104,9 @@ stripped_char (gunichar ch)
 	case G_UNICODE_SPACE_SEPARATOR:
 	default:
 		ch = g_unichar_tolower (ch);
-#if GLIB_CHECK_VERSION (2,29,90)
 		dlen = g_unichar_fully_decompose (ch, FALSE, decomp, 4);
-		if (dlen > 0) {
-#else
-		decomp = g_unicode_canonical_decomposition (ch, &dlen);
-		if (decomp != NULL) {
-#endif
+		if (dlen > 0)
 			retval = decomp[0];
-#if !GLIB_CHECK_VERSION (2,29,90)
-			g_free (decomp);
-#endif
-		}
 	}
 
 	return retval;
